/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to professionally license this library, the author
    can be reached at info@mohawksoft.com
*/ 
// #define TRACE_DEBUG 1
#define MSESSIONAPI_INTERNAL
#include "msessiond.h"
#include "msessionapi.h"

#define REQSIZE_DEF	2048
#define MREQ(conn)	(conn->preq->req)


// Converts a number of squential strings to an array.
// Strings are formatted as:
// string1\0string2\0string3\0\0
char ** StringsToArray(int count, char *strings)
{
	char **ret;
	int i;
	int cb;
	int n = count;
	char *p = strings;
	char **adds = (char **) alloca(sizeof(char *) * (n+1));
	int *lens = (int *) alloca(sizeof(int)*(n+1));

	// Find info about returned strings.
	for(i=0, cb=0; i < n; i++)
	{
		adds[i] = p;
		lens[i] = strlen(p)+1;
		cb+= lens[i]+1;
		p+= lens[i];
	}

	cb += (sizeof(char *) * (n+1));
	ret = (char **) phmalloc(cb);

	if(!ret)
		return NULL;

	p = (char *) &ret[n+1];

	for(i=0; i < n; i++)
	{
		ret[i] = p;
		memcpy(p, adds[i], lens[i]);
		p += lens[i];
	}
	ret[n]=NULL;

	return ret;
}
char ** Msession_conn::ResultToArray()
{
	return StringsToArray(m_reqb->req.param, m_reqb->req.datum);
}
char ** Msession_conn::ResultToPairs()
{
	return StringsToArray(m_reqb->req.param*2, m_reqb->req.datum);
}
char *Msession_conn::ResultToStr()
{
	if(m_reqb->req.len)
		return phmstrdup(m_reqb->req.datum);
	else
		return NULL;
}
#define RETRY_REQ

#define BADSOCK(pq)	((pq->req.stat == REQ_ERR) && \
	(pq->req.param == REQE_BADCONN || pq->req.param == REQE_NOSEND))
int Msession_conn::Request(int stat, char *session, char *name, char *value, int param)
{
	FormatRequest(&m_reqb, stat, session, name, value, param);
#ifndef RETRY_REQ
	m_errno = DoRequest(this, &m_reqb);
#else
	for (int retries=3; retries; retries--)
	{
		m_reqb->req.stat = stat;
		m_reqb->req.param = param;
		m_errno = DoRequest(this, &m_reqb);

		if(BADSOCK(m_reqb))
			ReopenReqConn(this);
		else
			return m_errno;

	}
#endif
	return m_errno;
}
int Msession_conn::RequestMulti(int stat, char *session, int n, char **pairs, int param)
{
	FormatRequestMulti(&m_reqb, stat, session, n, pairs,param);
	
#ifndef RETRY_REQ
	m_errno = DoRequest(this, &m_reqb);
#else
	for (int retries=3; retries; retries--)
	{
		m_reqb->req.stat = stat;
		m_reqb->req.param = param;
		m_errno = DoRequest(this, &m_reqb);
		if(BADSOCK(m_reqb))
			ReopenReqConn(this);
		else
			return m_errno;
	}
#endif
	return m_errno;
}
int Msession_conn::RequestStrings(int stat, char *session, int n, char **strings)
{
	int param;
	FormatRequestStrings(&m_reqb, stat, session, n, strings);
#ifndef RETRY_REQ
	m_errno = DoRequest(this, &m_reqb);
#else
	param = m_reqb->req.param;

	for (int retries=3; retries; retries--)
	{
		m_reqb->req.stat = stat;
		m_reqb->req.param = param;
		m_errno = DoRequest(this, &m_reqb);
		if(BADSOCK(m_reqb))
			ReopenReqConn(this);
		else
			return m_errno;
	}
#endif
	return m_errno;
}
Msession_conn::Msession_conn(char *host, int port) : MSock(port)
{
	m_reqb = AllocateRequestBuffer(REQSIZE_DEF);
	assert(m_reqb);

	if(!OpenReqSock(&m_reqb, this, host))
	{
		sprintf(m_reqb->req.datum, "Can not open connection to host \"%s\" on port %d", host, port);
		m_reqb->req.len = strlen(m_reqb->req.datum)+1;
		m_errno = REQ_ERR;
	}
	else
		m_errno = REQ_OK;
}

Msession_conn::~Msession_conn()
{
	CloseReqSock(&m_reqb,this);
	FreeRequestBuffer(m_reqb);
}

Boolean Msession_conn::create(char *session)
{
	TRACE_STRSTR("msession_create",session);
	Request(MREQ_CREATE, session, "", "",0);
	return StatOK();
}
Boolean Msession_conn::create_pairs(char *session, int count, char **pairs)
{
	TRACE_STRSTR("msession_create_pairs",session);
	RequestMulti(MREQ_CREATE, session, count, pairs,0);
	return StatOK();
}
Boolean Msession_conn::destroy(char *session)
{
	TRACE_STRSTR("msession_destroy",session);
	Request(MREQ_DROP, session, "", "",0);
	return StatOK();
}
int Msession_conn::set(char *session, char *name, char *value)
{
	TRACE_STRSTR("msession_set",session);
	Request(MREQ_SETVAL, session, name, value,0);
	return StatOK();
}
int Msession_conn::set_array(char *session, int count, char **pairs)
{
	TRACE_STRSTR("msession_set_array",session);
	RequestMulti(MREQ_SETVAL, session, count, pairs, 0);
	return StatOK();
}
int Msession_conn::get_int(char *session, char *name, int def)
{
	TRACE_STRSTR("msession_get_int",session);
	Request(MREQ_GETVAL, session, name, "",0);
	return StatOK() ? param() : def;
}
char *Msession_conn::get_text(char *session, char *name, char *def)
{
	TRACE_STRSTR("msession_get_text",session);
	Request(MREQ_GETVAL, session, name, "",0);
	return StatOK() ? ResultToStr() : def;
}
char * Msession_conn::uniq(int cbstr)
{
	TRACE_STR("msession_uniq");
	Request(MREQ_UNIQ, "", "", "",cbstr);
	return StatOK() ? ResultToStr() : NULL;
}
char * Msession_conn::randstr(int cbstr)
{
	TRACE_STR("msession_randstr");
	Request(MREQ_RANDSTR, "", "", "",cbstr);
	return StatOK() ? ResultToStr() : NULL;
}
char **Msession_conn::get_array(char *session)
{
	TRACE_STRSTR("msession_array",session);
	Request(MREQ_GETALL, session, "", "",0);
	return StatOK() ? ResultToPairs(): NULL;
}
char **Msession_conn::listvar(char *name)
{
	TRACE_STRSTR("msession_listvar",name);
	Request(MREQ_LISTVAR, "", name, "",0);
	return StatOK() ? ResultToPairs(): NULL;
}
char **Msession_conn::find(char *name, char *value)
{
	TRACE_STRSTR("msession_find",name);
	Request(MREQ_FIND, "",name,value,0);
	return StatOK() ? ResultToArray(): NULL;
}
char **Msession_conn::list()
{
	TRACE_STR("msession_list");
	Request(MREQ_LIST, "","","",0);
	return StatOK() ? ResultToArray(): NULL;
}
int Msession_conn::count()
{
	TRACE_FUNCT("msession_count");
	Request(MREQ_COUNT,"","","",0);
	return StatOK() ? param() : 0;
}
int Msession_conn::lock(char *session)
{
	TRACE_STRSTR("msession_lock",session);
	Request(MREQ_SLOCK,session,"","",0);
	return StatOK() ? param() : 0;
}
int Msession_conn::unlock(char *session, int key)
{
	TRACE_STRSTR("msession_unlock",session);
	Request(MREQ_SUNLOCK,session,"","",key);
	return StatOK();
}
Boolean Msession_conn::timeout(char *session, int timeout)
{
	TRACE_STRSTR("msession_timeout",session);
	Request(MREQ_TIMEOUT,session,"","",timeout);
	return StatOK();
}
int Msession_conn::inc_int(char *session, char *name)
{
	TRACE_STRSTR("msession_inc_int",session);
	Request(MREQ_INC, session, name, "",0);
	return StatOK() ? param() : 0;
}
char *Msession_conn::inc_text(char *session, char *name)
{
	TRACE_STRSTR("msession_inc_text",session);
	Request(MREQ_INC, session, name, "",0);
	return StatOK() ? ResultToStr() : NULL;
}
int Msession_conn::setdata(char *session, char *data)
{
	TRACE_STRSTR("msession_setdata",session);
	TRACE_STR(data);
	int len = strlen(data)+1;
	Request(MREQ_DATASET,session,"",data,len);
	return StatOK();
}
char *Msession_conn::getdata(char *session)
{
	TRACE_STRSTR("msession_getdata",session);
	Request(MREQ_DATAGET,session,"","",0);
	return StatOK() ? ResultToStr() : 0;
}
int Msession_conn::call_int(int n, char **strings)
{
	RequestStrings(MREQ_CALL, NULL, n, strings);
	return  StatOK() ? param() : 0;
}
char *Msession_conn::call_text(int n, char **strings)
{
	RequestStrings(MREQ_CALL, NULL, n, strings);
	return StatOK() ? ResultToStr() : 0;
}
int Msession_conn::ctl(char *session,int type)
{
	Request(MREQ_CALL, session, "", "", type);
	return  StatOK() ? param() : 0;
}
int Msession_conn::ctl_type(char *str)
{
        static char *parray[] =
        {       "EXIST", "TTL", "AGE", "TLA", "CTIME",
                "TOUCH", "NOW", NULL
        };
	for(int i=0; parray[i]; i++)
		if (strcasecmp(parray[i], str)==0)
			return i;
	return 0;
}
int Msession_conn::plugin_int(char *session, char *nparam, char *sparam, int xparam)
{
	Request(MREQ_PLUGIN, session, nparam, sparam, xparam);
	return  StatOK() ? param() : 0;
}
char *Msession_conn::plugin_text(char *session, char *nparam, char *sparam, int param)
{
	Request(MREQ_PLUGIN, session, nparam, sparam, param);
	return StatOK() ? ResultToStr() : NULL;
}
int Msession_conn::flush(int seconds)
{
	Request(MREQ_FLUSH, "", "", "", seconds);
	return StatOK();
}
int Msession_conn::admin(int fn, char *opt)
{
	Request(MREQ_ADMIN, opt, "", "",fn);
	return StatOK();
}
int Msession_conn::serialize(char *szparam)
{
	Request(MREQ_SERIALIZE, szparam, "", "", FALSE);
	return StatOK();
}
char *Msession_conn::exec(char *szcommand)
{
	Request(MREQ_EXEC,szcommand, "", "", 0);
	return StatOK() ? ResultToStr() : 0;
}
int Msession_conn::ping(void)
{
	Request(REQ_PING,"", "", "", 0);
	return StatOK();
}

/////////////////    C API   ////////////////
EXTAPI MSCONN msession_connect(char *host, int port)
{
	return new Msession_conn(host, port);

}
EXTAPI void msession_disconnect(MSCONN conn)
{
	delete conn;
}
EXTAPI Boolean msession_create(MSCONN conn, char *session)
{
	return conn->create(session);
}
EXTAPI Boolean msession_create_pairs(MSCONN conn, char *session, int count, char **pairs)
{
	return conn->create_pairs(session,count, pairs);
}
EXTAPI Boolean msession_destroy(MSCONN conn, char *session)
{
	return conn->destroy(session);
}
EXTAPI int msession_set(MSCONN conn, char *session, char *name, char *value)
{
	return conn->set(session, name, value);
}
EXTAPI int msession_set_array(MSCONN conn, char *session, int count, char **pairs)
{
	return conn->set_array(session, count,pairs);
}
EXTAPI int msession_get_int(MSCONN conn, char *session, char *name, int def)
{
	return conn->get_int(session, name, def);
}
EXTAPI char *msession_get_text(MSCONN conn, char *session, char *name, char *def)
{
	return conn->get_text(session,name, def);
}
EXTAPI char * msession_uniq(MSCONN conn, int cbstr)
{
	return conn->uniq(cbstr);
}
EXTAPI char * msession_randstr(MSCONN conn, int cbstr)
{
	return conn->randstr(cbstr);
}
EXTAPI char **msession_get_array(MSCONN conn, char *session)
{
	return conn->get_array(session);
}
EXTAPI char **msession_listvar(MSCONN conn, char *name)
{
	return conn->listvar(name);
}
EXTAPI char **msession_find(MSCONN conn, char *name, char *value)
{
	return conn->find(name, value);
}
EXTAPI char **msession_list(MSCONN conn)
{
	return conn->list();
}
EXTAPI int msession_count(MSCONN conn)
{
	return conn->count();
}
EXTAPI int msession_lock(MSCONN conn, char *session)
{
	return conn->lock(session);
}
EXTAPI int msession_unlock(MSCONN conn, char *session, int key)
{
	return conn->unlock(session, key);
}
EXTAPI Boolean msession_timeout(MSCONN conn, char *session, int timeout)
{
	return conn->timeout(session, timeout);
}
EXTAPI int msession_inc_int(MSCONN conn, char *session, char *name)
{
	return conn->inc_int(session, name);
}
EXTAPI char *msession_inc_text(MSCONN conn, char *session, char *name)
{
	return conn->inc_text(session,name);
}
EXTAPI int msession_setdata(MSCONN conn, char *session, char *data)
{
	return conn->setdata(session,data);
}
EXTAPI char *msession_getdata(MSCONN conn, char *session)
{
	return conn->getdata(session);
}
EXTAPI int msession_call_int(MSCONN conn, int n, char **strings)
{
	return conn->call_int(n, strings);
}
EXTAPI char *msession_call_text(MSCONN conn, int n, char **strings)
{
	return conn->call_text(n,strings);
}
EXTAPI int msession_ctl(MSCONN conn, char *session,int type)
{
	return conn->ctl(session,type);
}
EXTAPI int msession_ctl_type(char *str)
{
	return Msession_conn::ctl_type(str);
}
EXTAPI int msession_plugin_int(MSCONN conn, char *session, char *nparam, char *sparam, int param)
{
	return conn->plugin_int(session,nparam,sparam,param);
}
EXTAPI char *msession_plugin_text(MSCONN conn, char *session, char *nparam, char *sparam, int param)
{
	return conn->plugin_text(session,nparam,sparam,param);
}
EXTAPI int msession_errno(MSCONN conn)
{
	return conn->merrno();
}
EXTAPI char *msession_errtext(MSCONN conn)
{
	return conn->errtext();
}
EXTAPI int msession_flush(MSCONN conn, int seconds)
{
	return conn->flush(seconds);
}
EXTAPI int msession_serialize(MSCONN conn, char *szparam)
{
	return conn->admin(MADMIN_SERIALIZE,szparam);
}
EXTAPI int msession_loaddll(MSCONN conn, char *dll)
{
	return conn->admin(MADMIN_LOADDLL,dll);
}
EXTAPI int msession_shutdown(MSCONN conn, char *tim)
{
	return conn->admin(MADMIN_SHUTDOWN,tim);
}
EXTAPI int msession_setserial(MSCONN conn, char *serialize)
{
	return conn->admin(MADMIN_SETSERIAL,serialize);
}
EXTAPI int msession_setmaxthreads(MSCONN conn, char *maxt)
{
	return conn->admin(MADMIN_SETMAXTHREADS,maxt);
}
EXTAPI int msession_setverbose(MSCONN conn, char *verbose)
{
	return conn->admin(MADMIN_SETVERBOSE,verbose);
}
EXTAPI int msession_setautosave(MSCONN conn, char *autsave)
{
	return conn->admin(MADMIN_AUTOSAVE,autsave);
}
EXTAPI int msession_setautodrop(MSCONN conn, char *autdrop)
{
	return conn->admin(MADMIN_AUTODROP,autdrop);
}
EXTAPI int msession_status(MSCONN conn)
{
	return conn->admin(MADMIN_STATUS,"");
}
EXTAPI int msession_ping(MSCONN conn)
{
	return conn->ping();
}
EXTAPI char *msession_text(MSCONN conn)
{
	return conn->reqtext();
}
EXTAPI char *msession_exec(MSCONN conn, char *szcommand)
{
	return conn->exec(szcommand);
}
