/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to professionally license this library, the author
    can be reached at info@mohawksoft.com
*/ 
#ifndef _MSESSION_API_
#define _MSESSION_API_
#include "reqclient.h"

#ifdef __cplusplus
#include "Phoenix.h"
class Msession_conn : public MSock
{
	protected:
	REQB	*m_reqb;
	int	m_errno;
	public:
	Msession_conn(char *host, int port);
	~Msession_conn();
	int Request(int stat, char *session, char *name, char *value, int param);
	int RequestMulti(int stat, char *session, int n, char **pairs, int param);
	int RequestStrings(int stat, char *session, int n, char **strings);
	char *	ResultToStr();
	char ** ResultToArray();
	char ** ResultToPairs();
	Boolean create(char *session);
	Boolean create_pairs(char *session, int count, char **pairs);
	Boolean destroy(char *session);
	int set(char *session, char *name, char *value);
	int set_array(char *session, int count, char **pairs);
	int get_int(char *session, char *name, int def);
	char *get_text(char *session, char *name, char *def);
	char * uniq(int cbstr);
	char * randstr(int cbstr);
	char **get_array(char *session);
	char **listvar(char *name);
	char **find(char *name, char *value);
	char **list();
	int count();
	int lock(char *session);
	int unlock(char *session, int key);
	Boolean timeout(char *session, int timeout);
	int inc_int(char *session, char *name);
	char *inc_text(char *session, char *name);
	int setdata(char *session, char *data);
	char *getdata(char *session);
	int call_int(int n, char **strings);
	char *call_text(int n, char **strings);
	int ctl(char *session,int type);
	int plugin_int(char *session, char *nparam, char *sparam, int param);
	char *plugin_text(char *session, char *nparam, char *sparam, int param);
	int flush(int seconds);
	int admin(int fn, char *opt);
	int serialize(char *szparam);
	char *exec(char *szcommand);
	int ping(void);

	static int ctl_type(char *str);

	inline Boolean StatOK(void)	{return (m_reqb->req.stat == REQ_OK); }
	inline int param()		{return m_reqb->req.param; }
	inline int merrno()		{return m_errno;}
	inline char *errtext()		{return ReqbErr(m_reqb); }
	inline char *reqtext(void)	{return (m_reqb->req.len > 1) ? m_reqb->req.datum : NULL; }

};

typedef Msession_conn *MSCONN;

#ifdef WIN32
#define EXTAPI	extern "C" __declspec(dllexport) 
#else
#define EXTAPI	extern "C"
#endif

#else
typedef void * MSCONN;
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef MSESSIONAPI_INTERNAL
MSCONN 	msession_connect(char *host, int port);

/** Disconnects a session from an msession daemon. Returns nothing. */
void 	msession_disconnect(MSCONN conn);

/** Creates a new session. Returns TRUE is successful, or FALSE if an error occured.
An error occurs if the session already exists. msession_errno() will return REQE_DUPSESSION
if the session exists */

Boolean msession_create(MSCONN conn, char *session);

/** Behaves exactly like msession_create, except that initial data is passed to populate the session.
"pairs" is an array of string data. pairs[0] contains a name, pairs[1] contains its value, pairs[2]
contains the second name, pairs[3] containes the second value, and so on. */

Boolean msession_create_pairs(MSCONN conn, char *session, int count, char **pairs);

/** Destroys a session. Returns TRUE on success, FALSE on error. */
Boolean msession_destroy(MSCONN conn, char *session);

/** Sets a value in a session. Returns TRUE on success, FALSE on error */
int 	msession_set(MSCONN conn, char *session, char *name, char *value);

int msession_set_array(MSCONN conn, char *session, int count, char **pairs);
/** Gets a value in a session as an integer. Returns the value from the session, or
def from the argument list, if no value was found. */
int 	msession_get_int(MSCONN conn, char *session, char *name, int def);


/** Gets a value in a session as a string. Returns the value from the session, or
def from the argument list, if no value was found. */
char *	msession_get_text(MSCONN conn, char *session, char *name, char *def);


/** Create a new session with a guaranteed unique name. Returns a string of the name */
char * msession_uniq(MSCONN conn, int cbstr);

/** Create a pseudo-random alphanumeric string */
char * msession_randstr(MSCONN conn, int cbstr);

/** Gets all the name/value pairs from a session into a character array. The array is 
formatted as an array of string data. array[0] contains a name, array[1] contains its 
value, array[2] contains the second name, array[3] containes the second value, and so on.
Returns NULL on error. */

char ** msession_get_array(MSCONN conn, char *session);

/** Finds sessions which have the particular name/value property. Returns NULL
if none were found. */
char ** msession_find(MSCONN conn, char *name, char *value);

int msession_inc_int(MSCONN conn, char *session, char *name);
char *msession_inc_text(MSCONN conn, char *session, char *name);

/** Lists all active sessions. */
char ** msession_list(MSCONN conn);

/** Returns an associative array of session:var for all sessions which have
a setting of name. */
char **msession_listvar(MSCONN conn, char *name);

/** Returns the number of active sessions */
int 	msession_count(MSCONN conn);

/** Locks a session. Returns an integer key for the lock.
Locks remain exclusive for 30 seconds, after which someone else 
can lock the session. Locks are coperative, no enforcement
is attempted. */

int	msession_lock(MSCONN conn, char *session);

/** Unlocks a session */
int	msession_unlock(MSCONN conn, char *session, int key);


/** Sets the timeout of a session.
 * timeout = 0 means never timeout
 * timeout > 0 sets session expire time in seconds
 * timeout < 0 resets to default timeout
*/
Boolean msession_timeout(MSCONN conn, char *session, int timeout);

/** Sets the session data string, creates a session if one does not exist */
int msession_setdata(MSCONN conn, char *session, char *data);

/** Gets the session data string from a session */
char *msession_getdata(MSCONN conn, char *session);

/** Returns the last error.  */
int msession_errno(MSCONN conn);

/** Call a function plugin, return an int */
int msession_call_int(MSCONN conn, int n, char **strings);

/** Call a function plugin, return a char */
char *msession_call_text(MSCONN conn, int n, char **strings);

/** Call to get info about a session */
int msession_ctl(MSCONN conn, char *session, int type);

/** Convert a string ctl to a type for msession_ctl */
int msession_ctl_type(char *str);

/** Call the personality plugin escape function, return int */
int msession_plugin_int(MSCONN conn, char *session, char *nparam, char *sparam, int param);

/** Call the personality plugin escape function, return char * */
char *msession_plugin_text(MSCONN conn, char *session, char *nparam, char *sparam, int param);

int msession_flush(MSCONN conn, int seconds);
int msession_loaddll(MSCONN conn, char *dll);
int msession_shutdown(MSCONN conn, char *timeout);
int msession_serialize(MSCONN conn, char *szparam);
char * msession_exec(MSCONN conn, char *szparam);
int msession_setserial(MSCONN conn, char *serialize);
int msession_setmaxthreads(MSCONN conn, char *maxt);
int msession_setverbose(MSCONN conn, char *verbose);
int msession_setautosave(MSCONN conn, char *autsave);
int msession_setautodrop(MSCONN conn, char *autdrop);
int msession_ping(MSCONN conn);
int msession_status(MSCONN conn);

char *msession_text(MSCONN conn);

/** Returns a static character string describing an error. */
char *	msession_errtext(MSCONN conn);
#endif

#ifdef __cplusplus
}
#endif
#endif

