/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 2001 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to professionally license this library, the author
    can be reached at info@mohawksoft.com
*/ 

#include "postgres.h"
#include "access/heapam.h"
#include "catalog/catname.h"
#include "catalog/indexing.h"
#include "catalog/pg_proc.h"
#include "executor/executor.h"
#include "utils/fcache.h"
#include "utils/sets.h"
#include "utils/syscache.h"

#include <string.h>
#include "reqclient.h"


#ifndef MIN
#define MIN(a,b) ((a<b) ? a:b)
#endif
#ifndef REQSIZE_DEF
#define REQSIZE_DEF	1024
#endif

char	host[]	= "localhost";
int	port	= 8086;

#define MAX_RLAB 128

// Converts/copies postgres text to C string.
char *pgtostr(char *buffer, int cb, text *text)
{
	int len = VARSIZE(text)-VARHDRSZ;
	len = MIN(len,cb-1);
        memcpy(buffer, VARDATA(text), len);
        buffer[len]=0;
        return buffer;
}
text *strtopg(char *string)
{
	int slen = strlen(string);
	int cb = slen + VARHDRSZ;
	text * t = (text *)palloc(cb);
	if(t)
	{
		VARATT_SIZEP(t)=cb;
		strncpy(VARDATA(t),string, slen);
	}
	return t;
}

int msession_create(text *var)
{
	int stat;
	int err;
	char name[MAX_RLAB];
	REQB * preq= AllocateRequestBuffer(REQSIZE_DEF);

	char *session = pgtostr(name, sizeof(name), var);
	FormatRequest(&preq, REQ_CREATE, session, "none", "none",0);
	err = DoSingleRequest(host, port, &preq);
	stat = preq->req.stat;
	FreeRequestBuffer(preq);

	return stat == REQ_OK;
}
int msession_destroy(text *var)
{
	int stat;
	int err;
	char name[MAX_RLAB];
	REQB * preq= AllocateRequestBuffer(REQSIZE_DEF);

	char *session = pgtostr(name, sizeof(name), var);
	FormatRequest(&preq, REQ_DROP, session, "none", "none",0);
	err = DoSingleRequest(host, port, &preq);
	stat = preq->req.stat;
	FreeRequestBuffer(preq);
	return stat == REQ_OK;
}

int msession_set(text *vsession, text *vname, text *vvalue)
{
	int stat;
	char session[MAX_RLAB];
	char name[MAX_RLAB];
	char value[MAX_RLAB];
	REQB * preq= AllocateRequestBuffer(REQSIZE_DEF);

	pgtostr(session, sizeof(session), vsession);
	pgtostr(name, sizeof(name), vname);
	pgtostr(value, sizeof(value), vvalue);

	FormatRequest(&preq, REQ_SETVAL, session, name, value,0);
	DoSingleRequest(host,port,&preq);
	stat = preq->req.stat ;
	FreeRequestBuffer(preq);
	return stat == REQ_OK;
}
int4 msession_get_int(text *vsession, text *vname, int4 defval)
{
	int stat;
	char session[MAX_RLAB];
	char name[MAX_RLAB];
	REQB * preq= AllocateRequestBuffer(REQSIZE_DEF);

	pgtostr(session, sizeof(session), vsession);
	pgtostr(name, sizeof(name), vname);

	FormatRequest(&preq, REQ_GETVAL, session, name, "none",0);
	DoSingleRequest(host,port,&preq);
	stat = preq->req.stat;
	if(stat == REQ_OK)
		defval = preq->req.param;
	FreeRequestBuffer(preq);

	return defval;
}

text *msession_get_text(text *vsession, text *vname, text *defval)
{
	text *ret;
	char session[MAX_RLAB];
	char name[MAX_RLAB];
	REQB * preq= AllocateRequestBuffer(REQSIZE_DEF);

	pgtostr(session, sizeof(session), vsession);
	pgtostr(name, sizeof(name), vname);

	FormatRequest(&preq, REQ_GETVAL, session, name, "none",0);
	DoSingleRequest(host,port,&preq);

	if(preq->req.stat == REQ_OK)
		ret = strtopg(preq->req.datum);
	else
		ret = defval;
		
	FreeRequestBuffer(preq);
	return ret;
}

REQB *	g_preq;
int 	g_nndx=0;
int 	g_state=0;
char *	g_val;
int	g_ndx=0;

Datum msession_values_int(PG_FUNCTION_ARGS)
{
	int value=0;
	if(g_val)
		value = atoi(g_val);
	PG_RETURN_INT32(value);
}
Datum msession_values(PG_FUNCTION_ARGS)
{
	if(g_val)
	{
		text * result = strtopg(g_val);
		PG_RETURN_POINTER(result);
	}
	else
	{
		PG_RETURN_NULL();
	}
}
Datum msession_names(PG_FUNCTION_ARGS)
{
	if(!fcinfo->resultinfo)
	{
		elog(ERROR, "Not called with fcinfo");
		PG_RETURN_NULL();
	}	

	if(g_state == 0)
	{
		text *session;
		char szString[2000];
		int len ;
		g_ndx = 0;
		g_val = NULL;

    		session= PG_GETARG_TEXT_P(0);

		len = VARSIZE(session)-VARHDRSZ;

		if(!g_preq)
			g_preq= AllocateRequestBuffer(REQSIZE_DEF);

		memcpy(szString, VARDATA(session), len);
		szString[len]=0;
		FormatRequest(&g_preq, REQ_GETALL, szString, "", "",0);
		DoSingleRequest(host,port,&g_preq);
		
		if(g_preq->req.stat == REQ_OK )
		{
			g_state = g_preq->req.param; // Count of variable pairs	
			g_nndx=0;
		}
		else
		{
			g_state = 0;
			elog(NOTICE, "No values");
		}
	}

	if(g_ndx < g_state)
	{
		text *result;
		ReturnSetInfo *rsi; 
		char *str;
		rsi = (ReturnSetInfo *)fcinfo->resultinfo;
		/* Get name */
		str = &g_preq->req.datum[g_nndx];
		g_nndx += strlen(str)+1;
		/* Get value */
		g_val = (char *) &g_preq->req.datum[g_nndx];
		g_nndx += strlen(g_val)+1;
		g_ndx++;

		rsi->isDone = ExprMultipleResult;
		result = strtopg(str);
		PG_RETURN_POINTER(result);
	}
	else
	{
		ReturnSetInfo *rsi ;
		g_state=0;
		g_ndx=0;
		g_val = NULL;
		rsi = (ReturnSetInfo *)fcinfo->resultinfo;
		rsi->isDone = ExprEndResult ;
	}
	PG_RETURN_NULL();
}

PG_FUNCTION_INFO_V1(msession_names);
PG_FUNCTION_INFO_V1(msession_values);

Datum msession_find(PG_FUNCTION_ARGS)
{
	if(!fcinfo->resultinfo)
	{
		elog(ERROR, "Not called with fcinfo");
		PG_RETURN_NULL();
	}	

	if(g_state == 0)
	{
    		text * name= PG_GETARG_TEXT_P(0);
    		text * value= PG_GETARG_TEXT_P(1);

		int lenname = VARSIZE(name)-VARHDRSZ;
		char szname[lenname+1];

		int lenvalue = VARSIZE(value)-VARHDRSZ;
		char szvalue[lenvalue+1];

		memcpy(szname, VARDATA(name), lenname);
		memcpy(szvalue, VARDATA(value), lenvalue);

		szname[lenname]=0;
		szvalue[lenvalue]=0;

		FormatRequest(&g_preq, REQ_FIND, "none",szname,szvalue,0);
		DoSingleRequest(host,port,&g_preq);

		if(g_preq->req.stat == REQ_OK)
		{
			g_state = g_preq->req.param;
			g_nndx=0;
			g_val=NULL;
		}
		else
		{
			g_state=0;
			elog(NOTICE, "No values");
		}
	}

	if(g_ndx < g_state)
	{
		char *str;
		text *result;
		ReturnSetInfo *rsi = (ReturnSetInfo *)fcinfo->resultinfo;
		rsi->isDone = ExprMultipleResult;
		str = &g_preq->req.datum[g_nndx];
		g_nndx += strlen(str)+1;
		result = strtopg(str);
		PG_RETURN_POINTER(result);
	}
	else
	{
		ReturnSetInfo *rsi = (ReturnSetInfo *)fcinfo->resultinfo;
		rsi->isDone = ExprEndResult ;
	}
	PG_RETURN_NULL();
}

PG_FUNCTION_INFO_V1(msession_find);
