/*
    Mohawk Software Framework by Mohawk Software
    Copyright (C) 1998-2002 Mark L. Woodward
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
 
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.
 
    You should have received a copy of the GNU General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
    MA 02111-1307, USA
 
    If you want support or to professionally license this library, the author
    can be reached at info@mohawksoft.com
*/ 


#include "Phoenix.h"
#include "msessionapi.h"

typedef int (*FN) (MSCONN conn, char *opt);
typedef struct command
{
	char *	name;
	char *	opts;
	int	nopts;
	FN 	fn;
}COMMANDS;

int flush_str(MSCONN conn, char *seconds)
{
	int n = (seconds) ? atoi(seconds) : 0;
	return msession_flush(conn, n);
}


COMMANDS commands[]={
{"SHUTDOWN","[seconds]  \tShuts down the msession server.", 3, (FN)msession_shutdown},
{"LOADDLL","dllpath     \tLoads a DLL or shared library into the msession server",4,msession_loaddll},
{"SETSERIAL","string    \tSets the serialization string in the msession server",4,msession_setserial},
{"SETVERBOSE","number   \tSets the verbosity of msession logging",4,msession_setverbose},
{"SETMAXTHREADS","number\tSets the maximum number of threads",4,msession_setmaxthreads},
{"SERIALIZE","[string]  \tCall serialize function",3,msession_serialize},
{"FLUSH","[time]        \tCauses sessions to be flushed",4,flush_str},
{"AUTOSAVE","minutes    \tSets autosave interval",4,msession_setautosave},
{"AUTODROP","y/n        \tAllows/Disallows dropping persistent connections",4,msession_setautodrop},
{"PING","               \tChecks is msession server is working", 3, (FN)msession_ping},
{"STATUS","             \tGets status information from server", 3, (FN)msession_status},

{NULL,NULL,0, NULL}
};

int main(int argc, char **argv)
{
	int i;
	if(argc < 3 )
	{
		printf("Usage: %s host operation parameter\n", argv[0]);
		printf("Sends administration information to msession daemon\n");
		for(i=0; commands[i].name; i++)
			printf("\t%s %s\n", commands[i].name,commands[i].opts);
		exit(0);
	}

	int cbhost = strlen(argv[1])+1;
	char *host = (char *)alloca(cbhost);
	int port = HostPort(host, argv[1], 8086);

	InitPhoenix(argc, argv);

	for(i=0; commands[i].name; i++)
		if(strcasecmp(argv[2], commands[i].name)==0)
			break;

	if(commands[i].name == NULL)
	{
		printf("Command not recognized: %s\n", argv[2]);
		exit(0);
	}

	if(argc < commands[i].nopts)
	{
		printf("Not enough parameters for command\n");
		printf("\t%s %s\n", commands[i].name,commands[i].opts);
		
	}

	MSCONN conn = msession_connect(host,port);
	if(conn->StatOK())
	{
		if((commands[i].fn)(conn, argv[3]))
		{
			char *text = msession_text(conn);
			
			if(text)
				printf("%s\n", text);
			else
				printf("Success\n");

			msession_disconnect(conn);
			exit(0);
		}
	}
	printf("Failure: %s\n", msession_errtext(conn));
	msession_disconnect(conn);
}
